/*
PyPanel v0.9 - Lightweight panel/taskbar for X11 window managers
Copyright (c) 2003 Jon Gelo (ziljian@users.sourceforge.net)

This file is part of PyPanel.
PyPanel is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <Python.h>

#ifdef HAVE_XPM
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include "ppicon.xpm"
#endif

typedef unsigned long CARD32;

/*---------------------------------------------------------*/
static PyObject * _ppshade(PyObject *self, PyObject *args) {
/*---------------------------------------------------------*/
    CARD32 rmask, gmask, bmask;
    CARD32 *table, *rtable, *gtable, *btable;
    char *src, *trg;
    int i, size, bpp, bpl, w, h, shade, rsh, gsh, bsh;
    const int tint = 65535;
    unsigned int upper;

    if (!PyArg_ParseTuple(args, "s#iiii", &src, &size, &bpp, &w, &h, &shade))
        return NULL;
    
    trg = src;
    if (shade < 0) shade = 0;
    if (shade > 100) shade = 100;
    upper = (unsigned int)((((CARD32)tint)*((CARD32)shade))/100);
    
    switch (bpp) {
        case 16: {
            rmask   = 0xf800;
            gmask   = 0x07e0;
            bmask   = 0x001f;
            table   = (CARD32 *) malloc(sizeof(CARD32)*(32+64+32));
            rtable  = table;
            gtable  = table+32;
            btable  = table+32+64;
            rsh     = 11;
            gsh     = 5;
            bsh     = 0;
            bpl     = w*2;
            break;
        }
        case 32: {
            rmask   = 0xff0000;
            gmask   = 0x00ff00;
            bmask   = 0x0000ff;
            table   = (CARD32 *) malloc(sizeof(CARD32)*(256+256+256));
            rtable  = table;
            gtable  = table+256;
            btable  = table+256+256;
            rsh     = 16;
            gsh     = 8;
            bsh     = 0;
            bpl     = w*4;
            break;
        }
        default:
            return Py_BuildValue("s#", trg, size);
    }
    
    for (i = 0; i <= rmask>>rsh; i++) {
        CARD32 tmp;
        tmp = ((CARD32)i)*((CARD32)upper);
        rtable[i] = (tmp/tint)<<rsh;
    }
    for (i = 0; i <= gmask>>gsh; i++) {
        CARD32 tmp;
        tmp = ((CARD32)i)*((CARD32)upper);
        gtable[i] = (tmp/tint)<<gsh;
    }
    for (i = 0; i <= bmask>>bsh; i++) {
        CARD32 tmp;
        tmp = ((CARD32)i)*((CARD32)upper);
        btable[i] = (tmp/tint)<<bsh;
    }
    
    switch (bpp) {
        case 16: {
            unsigned short *p1, *pf, *p, *pl;
            p1 = (unsigned short *) trg;
            pf = (unsigned short *) (trg + h * bpl);
        
            while (p1 < pf) {
                p = p1;
                pl = p1 + w;
                for (; p < pl; p++) {
                    *p = rtable[(*p & rmask)>>11] |
                         gtable[(*p & gmask)>> 5] |
                         btable[(*p & bmask)];
                }
                p1 = (unsigned short *) ((char *) p1 + bpl);
            }
          break;
        }
        case 32: {
            CARD32 *p1, *pf, *p, *pl;
            p1 = (CARD32 *) trg;
            pf = (CARD32 *) (trg + h * bpl);
        
            while (p1 < pf) {
                p = p1;
                pl = p1 + w;
                for (; p < pl; p++) {
                    *p = rtable[(*p & rmask)>>16] |
                         gtable[(*p & gmask)>> 8] |
                         btable[(*p & bmask)]     |
                                (*p & ~0xffffff);
                }
                p1 = (CARD32 *) ((char *) p1 + bpl);
            }
            break;
        }
    }
    
    free(table);
    return Py_BuildValue("s#", trg, size);
}

/*--------------------------------------------------------*/
static PyObject * _ppicon(PyObject *self, PyObject *args) {
/*--------------------------------------------------------*/
    #ifdef HAVE_XPM
        Pixmap icon, mask, px;
        Window root;
        Display *dsp;
        int x, y, w, h, d, bw;
    
        if (!PyArg_ParseTuple(args, "l", &root))
            return NULL;
        
        dsp = XOpenDisplay(NULL);
        XpmCreatePixmapFromData(dsp, root, ppicon, &icon, &mask, NULL);
        XGetGeometry(dsp, icon, &px, &x, &y, &w, &h, &bw, &d);
        XFreePixmap(dsp, px);
        return Py_BuildValue("llii", icon, mask, w, h);
    #else
        Py_INCREF(Py_None);
        return Py_None;
    #endif
}

/*-------------------------------*/
static PyMethodDef PPMethods[] = {
/*-------------------------------*/
    {"ppshade",  _ppshade, METH_VARARGS, "PyPanel Shading Function"},
    {"ppicon",   _ppicon,  METH_VARARGS, "PyPanel Icon Function"},
    {NULL, NULL, 0, NULL}
};

/*------------------*/
/*  PyMODINIT_FUNC  */
void initppmodule() {
/*------------------*/
    (void)Py_InitModule("ppmodule", PPMethods);
}
